<?php
session_start();
// Easy-Tournament 1.1
// Started 20 March 2022
// Fixed some bugs with archiving and added default public - 30 June 2022

// ----- FILE HANDLING FUNCTIONS

// Write simple ini, a general way of saving files like tour.cfg
// $c is the array name
function writesimpleini($inifilename, $c) {
	$myfile = fopen($inifilename, "w");
	if (isset($c)) {
		if (is_array($c)) {
			foreach ($c as $k => $v)
				fwrite($myfile, $k . "=" . $v . chr(10));
		}
	}
	fclose($myfile);
}

// Read simple ini, because parse_ini_file makes exceptions, such as
// not liking exclamation marks
function readsimpleini($inifilename) {
	$myfile = fopen($inifilename, "r");
	while ($s = fgets($myfile)) {
		$n = strpos($s, "=");
		$l = strlen($s);
		if ($n <> false) {
			$k = substr($s, 0, $n);
			$v = substr($s, $n + 1, $l - $n - 2);
			$c[$k] = rtrim($v);
		}
	}
	fclose($myfile);
	
	if (isset($c))
		return $c;
	else
		return false;
}

// Write 2D ini file (with headers)
function write2dini($inifilename, $c) {
	$myfile = fopen($inifilename, "w");
	foreach ($c as $k => $v) {
		if (is_array($c[$k])) {
			fwrite($myfile, "[" . $k . "]" . chr(10));
			foreach ($c[$k] as $kk => $vv)
				fwrite($myfile, $kk . "=" . $vv . chr(10));
			fwrite($myfile, chr(10));
		} else
			fwrite($myfile, $k . "=" . $v . chr(10));
	}
	fclose($myfile);
}

// Read 2D ini file (with headers)
function read2dini($inifilename) {
	$myfile = fopen($inifilename, "r");
	$curheader = false;
	while ($s = fgets($myfile)) {
		$l = strlen($s);
		// See if it's a header
		if (strcmp(substr($s, 0, 1), "[") == 0 && strcmp(substr($s, $l - 2, 1), "]") == 0)
			$curheader = substr($s, 1, strlen($s) - 3);
		else {
			$n = strpos($s, "=");
			if ($n <> false) {
				$k = substr($s, 0, $n);
				$v = substr($s, $n + 1, $l - $n - 2);
				if ($curheader == false)
					$c[$k] = rtrim($v);
				else
					$c[$curheader][$k] = rtrim($v);
			}
		}
	}
	fclose($myfile);
	
	if (isset($c))
		return $c;
	else
		return false;
}

// Get base user file name from user name
function userbase($user)
{
	$t = "";
	for ($i = 0; $i < strlen($user); $i++)
	{
		$s = substr($user, $i, 1);
		if ($s == " ")
			$t = $t . "_";
		elseif (ord($s) > 31)
			$t = $t . $s;
	}
	return strtolower($t);
}

// Convert replay file length to lap time
function LengthToLap($lap)
{
    $tics = $lap - 1848;
    $m = floor($tics / 1200);
    $s = ($tics - $m * 1200) / 20;
    if ($s < 10) $s = "0" . $s;
    if (strlen($s) == 2)
		$s = $s . ".00";
	elseif (strpos($s, ".") == strlen($s) - 2)
		$s .= "0";
		
    return($m . ":" . $s);
}

// Read metadata from a track
function readtrackmetadata($trk) {
	$myfile = fopen($trk, "r");
	fseek($myfile, 1802);
	$smdf = fread($myfile, 4);
	if (feof($myfile)) {
		fclose($myfile);
		return false;
	}
	if (strcmp($smdf, "smdf") == 0) {
		$tag = fread($myfile, 4); // skip four bytes
		while (!feof($myfile)) {
			$tag = fread($myfile, 4);
			$b1 = fread($myfile, 1);
			$b2 = fread($myfile, 1);
			$l = ord($b1) + 256 * ord($b2);
			if ($l <> 0) $v = fread($myfile, $l);
			$r[$tag] = $v;
		}
		
	}
	fclose($myfile);
	if (!isset($r)) $r = false;
	return $r;
}

function DisplaySB($sb) {
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	
	?>
	<table class="scoreboard" style="margin-left:auto;margin-right:auto;">
	<tr class="sbheader">
		<th>#</th>
		<th>Posted</th>
		<th>Racer</th>
		<th>Car</th>
		<th>Lap Time</th>
		<th>Status</th>
	</tr>
	<?php
	
	for ($i = 1; $i < 255; $i++) {
		if (!isset($sb[$i])) break;
		
		if (isset($sb[$i]["lap"]))
			$lap = LengthToLap($sb[$i]["lap"]);
		else
			$lap = "Unknown";
		
		if ($sb[$i]["public"] == "yes")
			$dl = ' <a href="' . $sb[$i]["rpl"] . '">[Download]</a>';
		elseif ($isadmin)
			$dl = '* <a href="idl.php?replay=' . $sb[$i]["rpl"] . '">[Download]</a>';
		else
			$dl = '';
			
		$sb[$i]["visible"] = $sb[$i]["visible"] ?? "yes";
		
		if ($isadmin) {
			$dl .= ' <a href="' . basename(__FILE__) . '?admcmd=v&row=' . $i. '">[V]</a>';
			$dl .= ' <a href="' . basename(__FILE__) . '?admcmd=r&row=' . $i. '">[R]</a>';
			$dl .= ' <a href="' . basename(__FILE__) . '?admcmd=p&row=' . $i. '">[P]</a>';
			$dl .= ' <a href="' . basename(__FILE__) . '?admcmd=i&row=' . $i. '">[I]</a>';
			$dl .= ' <a href="' . basename(__FILE__) . '?admcmd=d&row=' . $i. '">[D]</a>';
		}
		
		if ($sb[$i]["visible"] != "no" | $isadmin) {
			if ($sb[$i]["visible"] == "no")
				echo '<tr class="sbhiddenrow">';
			else
				echo '<tr class="sbrow">';
			echo '<td style="text-align:right;">' . $i . '</td>';
			echo '<td>' . ($sb[$i]["posted"] ?? "Uknown") . ' ' . ($sb[$i]["posted-t"] ?? "") . '</td>';
			echo '<td>' . ($sb[$i]["name"] ?? "Uknown") . '</td>';
			echo '<td>' . ($sb[$i]["carid"] ?? "Uknown") . '</td>';
			echo '<td style="text-align:right;">' . $lap . '</td>';
			echo '<td>' . ($sb[$i]["status"] ?? "Uknown") . $dl . '</td>';
			echo '</tr>';
		}
	}
	
	echo "</table>";
}

function ReplayPostForm() {
	$loggedin = (($_SESSION["user"]["name"] ?? "") <> "");
	if (!$loggedin) return true;
	
	?>
	<h2>Post a Replay</h2>
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	<input type="file" name="UpReplay" accept=".rpl" id="UpReplay" required>
	<input type="submit" value="Upload Replay" name="post-replay">
	</form>
	<?php
}

// Push a new lap RPL into the scoreboard file $sb
function PushLap($newlap, $sb) {
	$newlap["status"] = "pending";
	
	if (!file_exists($sb)) {
		$sc[1] = $newlap;
		write2dini($sb, $sc);
		return;
	}
	
	$sc = read2dini($sb);
	$pushed = false; $i = 1; $j = 1;
	while (isset($sc[$i])) {	// While there are still laps in the scoreboard
		if ($pushed == false && $newlap["lap"] < $sc[$i]["lap"]) {
			$sc2[$j] = $newlap;
			$j++; $pushed = true;
		}
		
		// If it's the same racer, the original lap is only to be
		// pushed back into the scoreboard if the old one is not
		// pending verification
		if (strcasecmp($sc[$i]["name"], $newlap["name"]) == 0 &&
			$sc[$i]["carid"] == $newlap["carid"]) {
			if ($sc[$i]["status"] == "pending" || $sc[$i]["status"] == "rejected")
				$sc[$i]["competing"] = "no";

			$sc2[$j] = $sc[$i];
			$i++; $j++;
		} else {	// If not the same racer, always push
			$sc2[$j] = $sc[$i];
			$i++; $j++;
		}
	}
	if (!$pushed) {
		$sc2[$j] = $newlap;
	}
	write2dini($sb, $sc2);
}

// Delete a lap from the scoreboard file $sb
function PopLap($num, $sb) {
	$sc = read2dini($sb);
	$i = 1; $j = 1;
	while (isset($sc[$i])) {
		if ($i <> $num) {
			$sc2[$j] = $sc[$i];
			$j++;
		}
		$i++;
	}
	write2dini($sb, $sc2);
}

function ActuallyUploadReplay() {
	global $tour;
	global $admin;
	
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	$loggedin = (($_SESSION["user"]["name"] ?? "") <> "");
	
	if (!$loggedin) return "Not logged in";
	
	$OKtoUpload = true; $ruerror = "";
	
	$fsize = $_FILES["UpReplay"]["size"];
	if ($fsize > 13828 || $fsize <= 1828) {
		$OKtoUpload = false;
		$ruerror = "File is too large or too small";
	}
	
	$myfile = fopen($_FILES["UpReplay"]["tmp_name"], "r");
	$rplcarid = fread($myfile, 4);
	if (strpos(strtoupper($tour["carsallowed"]), strtoupper($rplcarid)) === false) {
		$OKtoUpload = false;
		$ruerror = "Car not allowed in this race";
	}
	fseek($myfile, 26);
			
	// Make sure the track is the same too
	$rpltrack = fread($myfile, 1801);
	$myotherfile = fopen($tour["trackfile"], "r");
	$tracktrack = fread($myotherfile, 1801);
	fclose($myotherfile);
	if ($rpltrack <> $tracktrack) {
		$OKtoUpload = false;
		$ruerror = "Replay was not raced on this track";
	}
	unset($rpltrack); unset($tracktrack);
			
	// Verify that the replay ends in a "silent" second
	$fs = filesize($_FILES["UpReplay"]["tmp_name"]);
	fseek($myfile, $fs - 20);
	$last2nd = fread($myfile, 20);
	for ($i = 0; $i < 20; $i++) {
		if (ord(substr($last2nd, $i, 1))) {
			$OKtoUpload = false;
			$ruerror = "Replay ends abruptly. It appears to not reach the finish line";
			break;
		}
	}
	fclose($myfile);
	
	if ($OKtoUpload) {
		// Get and update counter
		if (file_exists("replay-counter.dat"))
			$n = (int) file_get_contents("replay-counter.dat");
		else
			$n = 0;
		
		$rname = rtrim($admin["file-prefix"] ?? "") . $n . ".rpl";
		
		file_put_contents("replay-counter.dat", $n + 1);
		
		$default_public = $admin["default-public"] ?? "no";
		if ($default_public <> "no") {
			$default_public = "yes";
			$destdir = "";
		} else
			$destdir = "private/";
			
		if (!move_uploaded_file($_FILES["UpReplay"]["tmp_name"], $destdir . $rname))
			$ruerror = "Failed to upload replay";
		else {
			// Add replay to the scoreboard
			$myfile = fopen($destdir . $rname, "r");
			$rep["carid"] = fread($myfile, 4);
			fclose($myfile);
			$rep["name"] = $_SESSION["user"]["name"] ?? "Anonymous";
			$rep["lap"] = $fsize;
			$rep["rpl"] = rtrim($admin["file-prefix"] ?? "") . $n . ".rpl";
			$rep["posted"] = gmdate("Y-m-d");
			$rep["posted-t"] = gmdate("H:i:s");
			$rep["status"] = "pending";
			$rep["public"] = $default_public;
			PushLap($rep, $tour["scoreboard"]);
		}
	}
	
	echo $ruerror;
	return $ruerror;
}

function GenerateTrackImage() {
	global $tour;
	
	$im = imagecreatetruecolor(660, 660);
	$ter = imagecreatefrompng("./graphics/terrain.png");
	$telem = imagecreatefrompng("./graphics/trkelems.png");
	//$tile = imagecreatetruecolor(22, 22);
	
	$myfile = fopen($tour["trackfile"], "r");
	
	// Copy the terrain
	fseek($myfile, 901);
	for ($y = 0; $y < 30; $y++) {
		for ($x = 0; $x < 30; $x++) {
			$l[$x][$y] = ord(fread($myfile, 1));
			if ($l[$x][$y] > 19) $l[$x][$y] = 19;
			
			imagecopy($im, $ter, 22 * $x, 22 * $y, 0, 22 * $l[$x][$y], 22, 22);
		}
	}
	
	// Copy the track
	fseek($myfile, 0);
	for ($y = 29; $y >= 0; $y--) {
		for ($x = 0; $x < 30; $x++) {
			$t[$x][$y] = ord(fread($myfile, 1));
		}
	}
	
	for ($y = 0; $y < 30; $y ++) {
		for ($x = 0; $x < 30; $x++) {
			switch ($t[$x][$y]) {
				case 253:
					$o = $t[$x - 1][$y - 1];
					$sx = 2 * ($o % 16) + 1;
					$sy = 1 + (2 * (int) ($o / 16));
					break;
				case 254:
					$o = $t[$x][$y - 1];
					$sx = 2 * ($o % 16);
					$sy = 1 + 2 * ((int) ($o / 16));
					break;
				case 255:
					$o = $t[$x - 1][$y];
					$sx = 2 * ($o % 16) + 1;
					$sy = 2 * (int) ($o / 16);
					break;
				default:
					$o = $t[$x][$y];
					$sx = 2 * ($o % 16);
					$sy = 2 * (int) ($o / 16);
					break;
			}
			
			// Straightways on a slope have side bars
			if ($l[$x][$y] >= 7 && $l[$x][$y] <= 10) {
				if ($o == 4 || $o == 5 || $o == 14 || $o == 15 || $o == 0x18 || $o == 0x19)
					$sy += 1;
			}
			
			// Tunnel connections
			if ($o == 0x42) {
				$up = $t[$x][$y - 1] ?? 0;
				$down = $t[$x][$y + 1] ?? 0;
				
				if ($up == 0x42) $sy += 1;
				if ($down == 0x42) $sx += 1;	
			}
			if ($o == 0x43) {
				$left = $t[$x - 1][$y] ?? 0;
				$right = $t[$x + 1][$y] ?? 0;
				
				if ($left == 0x43) $sy += 1;
				if ($right == 0x43) $sx += 1;
			}
			
			imagecopy($im, $telem, 22 * $x, 22 * $y, 22 * $sx, 22 * $sy, 22, 22);
		}
	}
	
	unset($t); unset($l);
	fclose($myfile);
	
	imagepng($im, "track-image-" . $tour["track-image-number"] . ".png");
	
	imagedestroy($telem);
	//imagedestroy($tile);
	imagedestroy($ter);
	imagedestroy($im);
}

function ArchiveCurrentRace() {
	// Prepare directories
	if (!file_exists("races")) mkdir("races");
	if (file_exists("race-counter.dat"))
		$racenumber = file_get_contents("race-counter.dat");
	else
		$racenumber = 0;
	
	$season = gmdate("Y");
	if (strlen($racenumber) == 1) $racenumber = "0" . $racenumber;
	
	// Still in the same season?
	if (file_exists("races/" . $season . "-" . $racenumber . "/race.sb"))
		$racenumber += 1;
	else
		$racenumber = 1;
		
	if (strlen($racenumber) == 1) $racenumber = "0" . $racenumber;
	
	// Create new race directory
	$racedir = "races/" . $season . "-" . $racenumber;
	mkdir($racedir);
	
	// Move basic files
	copy("tour.cfg", $racedir . "/race.cfg");
	rename($tour["scoreboard"], $racedir . "/race.sb");
	rename($tour["trackfile"], $racedir . "/race.trk");
	
	// Move replays
	$rep = glob("*.rpl");
	foreach ($rep as $replay)
		rename($replay, $racedir . "/" . $replay);
	
	// Update current race number
	file_put_contents("race-counter.dat", $racenumber);
}

function ActuallyUploadTrack() {
	global $tour;
	
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	$loggedin = (($_SESSION["user"]["name"] ?? "") <> "");
	
	if (!$loggedin) return "Not logged in";
	
	$OKtoUpload = true; $ruerror = "";
	
	$fsize = $_FILES["UpTrack"]["size"];
	if ($fsize > 13828 || $fsize < 1802) {
		$OKtoUpload = false;
		$ruerror = "File is too large or too small";
	}
	
	if ($OKtoUpload) {
		// Remove old track
		unlink($tour["trackfile"]);
				
		if (!move_uploaded_file($_FILES["UpTrack"]["tmp_name"], $tour["trackfile"]))
			$ruerror = "Failed to upload track";
		else {
			// Remove old scoreboard
			$sb = trim($tour["scoreboard"]);
			if (file_exists($sb)) unlink($sb);
									
			// Remove old track image
			$image = "track-image-" . $tour["track-image-number"] . ".png";
			if (file_exists($image)) unlink($image);
			
			// Update image
			$tour["track-image-number"] = $tour["track-image-number"] + 1;
			GenerateTrackImage();
			
			// Update track information
			$tmd = readtrackmetadata($tour["trackfile"]);
			$tour["tracktitle"] = $tmd["Titl"] ?? $tour["trackfile"];
			$tour["trackauthor"] = $tmd["Autr"] ?? "Unknown";
			$today = gmdate("Y-m-d");
			$tour["startdate"] = date("Y-m-d", strtotime($today . " +1 day"));
			$tour["deadline"] = date("Y-m-d", strtotime($today . " +21 days"));
			writesimpleini("tour.cfg", $tour);
			
			// Remove all replays
			$fl = glob("*.rpl");
			foreach ($fl as $thisfile) unlink($thisfile);
			unset($fl); $fl = glob("private/*.rpl");
			foreach ($fl as $thisfile) unlink($thisfile);
			unset($fl);
		}
	}
	
	return $ruerror;
}

function menu() {
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	$loggedin = (($_SESSION["user"]["name"] ?? "") <> "");
	$op = '<td style="width:100px;text-align:center;"><a href="' . basename(__FILE__);
	echo '<table style="margin-left:auto;margin-right:auto;"><tr>';
	echo $op . '">HOME</a></td>';
	echo $op . '?page=rules">RULES</a></td>';
	if ($loggedin)
		echo $op . '?page=profile">PROFILE</a></td>';
	if ($isadmin)
		echo $op . '?page=admin">ADMIN</a></td>';
	if ($loggedin)
		echo $op . '?page=logout">LOG OUT</a></td>';
	else {
		echo $op . '?page=login">LOG IN</a></td>';
		echo $op . '?page=signup">SIGN UP</a></td>';
	}
	echo '</tr></table>';
}

function login_page() {
	?>
	<h2>Log in</h2>
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	User name<br>
	<input type="text" id="username" name="username" maxlength="32"><br>
	Password<br>
	<input type="password" id="password" name="password" maxlength="32"><br>
	<br>
	<input type="submit" name="login_button" value="Log in">
	</form>
	<?php
}

function signup_page() {
	?>
	<h2>Sign up</h2>
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	User name<br>
	<input type="text" id="username" name="username" maxlength="32"><br>
	Password<br>
	<input type="password" id="password" name="password" maxlength="32"><br>
	Repeat password<br>
	<input type="password" id="password2" name="password2" maxlength="32"><br>
	<br>
	<input type="submit" name="signup_button" value="Sign up">
	</form>
	<?php
}

function profile_page() {
	if (isset($_SESSION["user"]["name"])) {
		?>
		<h2>Profile - <?=$_SESSION["user"]["name"]?></h2>
		<br>
		<fieldset style="width:300px;margin-left:auto;margin-right:auto;">
		<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
		<?php
		if ($_SESSION["user"]["name"] == "admin") { ?>
			User name<br>
			<input type="text" id="username" name="username" value="admin" maxlength="32"><br>
			<?php
		} else { ?>
			<input type="hidden" id="username" name="username" value="<?=$_SESSION["user"]["name"]?>"><br>
			<?php
		} ?>
		Old password<br>
		<input type="password" id="oldpass" name="oldpass" maxlength="32"><br>
		New password<br>
		<input type="password" id="newpass" name="newpass" maxlength="32"><br>
		<br>
		<input type="submit" name="changepass_button" value="Change password">
		</form>
		</fieldset>
		<?php
	} else
		echo "<p>Forbidden</p>";
}

function rules_page() {
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	
	echo "<h2>Rules</h2>";
	
	if (file_exists("rules.html"))
		$rules = file_get_contents("rules.html");
	else
		$rules = "There are no rules. You can do whatever you want!";
	
	if ($isadmin) {
		$rules = htmlspecialchars($rules);
		?>
		<form action="<?=basename(__FILE__) . '?page=rules'?>" method="post" enctype="multipart/form-data">
		<textarea style="width:640px;height:480px;font-size:large;" name="rules-content"><?=$rules?></textarea>
		<br>
		<input type="submit" name="save-rules" value="Save Rules">
		</form>
		<?php
	} else
		echo $rules;
}

function admin_page() {
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	
	if (!$isadmin) {
		echo "<p>Forbidden</p>";
		return;
	}
	
	$tourfile = htmlspecialchars(file_get_contents("tour.cfg"));
	$adminfile = htmlspecialchars(file_get_contents("private/admin.cfg"));
	$cssfile = htmlspecialchars(file_get_contents("easytour.css"));
	?>
	<h2>Administration Panel</h2>
	<br>
	
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	<h3>Change current track</h3>
	<input type="file" name="UpTrack" accept=".trk" id="UpTrack" required>
	<input type="submit" value="Upload Track" name="post-track">
	</form>
	<p>Warning: Uploading a new track will cause the previous track file
	to be deleted from the tournament site, the track image to be updated
	and the scorebard to be cleared as well!  All replays will be removed
	from the server!  Make a backup if you want to keep this information.</p>
	<br><br>
	
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	
	<h3>Edit configuration files</h3>
	<table><tr><td>
	<textarea style="width:320px;height:200px;" name="tour-content"><?=$tourfile?></textarea>
	<br>
	<input type="submit" name="save-tour" value="Save tour.cfg">
	</td>
	<td>
	<textarea style="width:320px;height:200px;" name="admin-content"><?=$adminfile?></textarea>
	<br>
	<input type="submit" name="save-admin" value="Save admin.cfg">
	</td>
	<td>
	<textarea style="width:320px;height:200px;" name="css-content"><?=$cssfile?></textarea>
	<br>
	<input type="submit" name="save-css" value="Save CSS">
	</td>
	</tr></table>
	</form>
	<br>
	
	<h3>Race archive</h3>
	<?php
	if (file_exists($tour["scoreboard"])) {
		if (count(glob("private/*.rpl")) == 0) {
	?>
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	<input type="submit" value="Archive Current Race" name="archive-race">
	</form>
	<?php
		} else {
		?><p>There are unpublished replays!</p><?php
		}
	} else { ?>
	<p>No scoreboard. Nothing to archive</p>
	<?php } ?>
	<br>
	
	<h3>Load website images</h3>
	<table style="margin-left:auto;margin-right:auto;"><tr><td>
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	<input type="file" name="UpFavIcon" accept="image/*" id="UpFavIcon" required>
	<br><input type="submit" value="Upload Favicon" name="upload-favicon">
	</form>
	</td><td>
	<form action="<?=basename(__FILE__)?>" method="post" enctype="multipart/form-data">
	<input type="file" name="UpTopImage" accept="image/*" id="UpTopImage" required>
	<br><input type="submit" value="Upload Top Image" name="upload-topimage">
	</form>
	</td></tr></table>
	<?php
	
}

function home_page() {
	global $tour;
	
	$curdate = gmdate("Y-m-d");
	if ($curdate >= $tour["startdate"] && $curdate < $tour["deadline"])
		$racestatus = "Running";
	elseif ($curdate < $tour["startdate"])
		$racestatus = "Not yet started";
	else
		$racestatus = "Finished";
	
	
	$dl = date_create($tour["deadline"]);
	$dl = date_sub($dl, date_interval_create_from_date_string("1 minute"));
	$dl = date_format($dl, "Y-m-d H:i") . " UTC<br />";
	
	?>
	<h2>Home page</h2>
	<table class="trackinfo" style="margin-left:auto;margin-right:auto;">
	<tr><td>Current track</td><td><?=$tour["tracktitle"]?> <a href="<?=$tour["trackfile"]?>">[Download]</a></td></tr>
	<tr><td>Track author</td><td><?=$tour["trackauthor"]?></td></tr>
	<tr><td>Car</td><td><?=$tour["carsallowed"]?></td></tr>
	<tr><td>Race start</td><td><?=$tour["startdate"]?></td></tr>
	<tr><td>Race deadline</td><td><?=$dl?></td></tr>
	<tr><td>Status</td><td><?=$racestatus?></td></tr>
	</table>
	<br>
	<?php
	
	if (isset($tour["track-image-number"])) {
		$image = "track-image-" . $tour["track-image-number"] . ".png";
		echo '<img src="' . $image . '" style="width:640px;"><br>';
	}
	
	echo '<br>';	
	$tour["scoreboard"] = trim($tour["scoreboard"]);
	if (file_exists($tour["scoreboard"])) {
		$sb = read2dini($tour["scoreboard"]);
		DisplaySB($sb);
	} else
		echo "<p>No scoreboard yet!</p>";

	echo '<br>';
	if ($racestatus == "Running") ReplayPostForm();
}

function LogIn($uname, $pass) {
	$u = userbase($uname);
	$p = hash("snefru256", $pass);
	if (file_exists("private/" . $u . ".user")) {
		if (isset($_SESSION["user"])) unset($_SESSION["user"]);
		$udata = readsimpleini("private/" . $u . ".user");
		if ($udata["password"] == $p)
			$_SESSION["user"] = $udata;
		else {
			echo "<p>Wrong password!</p>";
			return 1;
		}
	} else {
		echo "<p>User does not exist!</p>";
		return 2;
	}
	return 0;
}

function SignUp($uname, $pass1, $pass2) {
	if ($pass1 <> $pass2) {
		echo "<p>Passwords mismatch</p>";
		return 1;
	} elseif (strlen($pass1) < 8) {
		echo "<p>Password must be at least 8 characters long</p>";
		return 2;
	} elseif (strlen($uname) < 3) {
		echo "<p>User name must be at least 3 characters long</p>";
		return 3;
	} elseif (strlen(userbase($uname)) < 3) {
		echo "<p>User name is inadequate</p>";
		return 4;
	} elseif (file_exists("private/" . userbase($uname) . ".user")) {
		echo "<p>User name already taken</p>";
		return 5;
	}
	
	$udata["name"] = $uname;
	$udata["password"] = hash("snefru256", $pass1);
	writesimpleini("private/" . userbase($uname) . ".user", $udata);
	echo "<p>Successfully signed up. You need to log in now</p>";
	return 0;
}

function ChangePassword($uname, $oldpass, $newpass) {
	if (!file_exists("private/" . userbase($uname) . ".user")) {
		echo "<p>User name does not exist</p>";
		return 4;
	}
	
	$udata = readsimpleini("private/" . userbase($uname) . ".user");
	
	if (strlen($newpass) < 8) {
		echo "<p>Password must be at least 8 characters long</p>";
		return 2;
	} elseif ($_SESSION["user"]["name"] <> "admin" && userbase($uname) <> userbase($_SESSION["user"]["name"])) {
		echo "<p>Not allowed to change password</p>";
		return 6;
	} elseif (hash("snefru256", $oldpass) <> $udata["password"] && $_SESSION["user"]["name"] <> "admin") {
		echo "<p>Wrong password</p>";
		return 1;
	}
	
	$udata["password"] = hash("snefru256", $newpass);
	writesimpleini("private/" . userbase($uname) . ".user", $udata);
	if (userbase($_SESSION["user"]["name"]) == userbase($uname))
		$_SESSION["user"]["password"] = $udata["password"];
	
	return 0;
}

function reactions() {
	global $tour;
	$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
	
	if (isset($_POST["login_button"]))
		LogIn($_POST["username"], $_POST["password"]);
	elseif (isset($_POST["signup_button"]))
		SignUp($_POST["username"], $_POST["password"], $_POST["password2"]);
	elseif (isset($_POST["changepass_button"]))
		ChangePassword($_POST["username"], $_POST["oldpass"], $_POST["newpass"]);
	
	if (isset($_POST["post-replay"])) ActuallyUploadReplay();
	
	if ($isadmin) {
		if (isset($_POST["save-tour"])) {
			file_put_contents("tour.cfg", $_POST["tour-content"]);
		} elseif (isset($_POST["save-admin"])) {
			file_put_contents("private/admin.cfg", $_POST["admin-content"]);
		} elseif (isset($_POST["save-css"])) {
			file_put_contents("easytour.css", $_POST["css-content"]);
		} elseif (isset($_POST["post-track"])) {
			ActuallyUploadTrack();
		} elseif (isset($_POST["save-rules"])) {
			file_put_contents("rules.html", $_POST["rules-content"]);
		} elseif (isset($_POST["upload-favicon"])) {
			move_uploaded_file($_FILES["UpFavIcon"]["tmp_name"], "favicon.ico");
		} elseif (isset($_POST["upload-topimage"])) {
			move_uploaded_file($_FILES["UpTopImage"]["tmp_name"], "topimage.png");
		} elseif (isset($_POST["archive-race"])) {
			ArchiveCurrentRace();
		}
		
		if (file_exists($tour["scoreboard"]))
			$sb = read2dini($tour["scoreboard"]);
			
		if (isset($_GET["admcmd"])) {
			$row = $_GET["row"];
			switch ($_GET["admcmd"]) {
				case "v":
					if ($sb[$row]["status"] == "verified")
						$sb[$row]["status"] = "pending";
					else
						$sb[$row]["status"] = "verified";
					
					write2dini($tour["scoreboard"], $sb);
					break;
				case "r":
					if ($sb[$row]["status"] == "rejected")
						$sb[$row]["status"] = "pending";
					else
						$sb[$row]["status"] = "rejected";
					
					write2dini($tour["scoreboard"], $sb);
					break;
				case "p":
					if ($sb[$row]["public"] == "yes") {
						rename($sb[$row]["rpl"], "private/" . $sb[$row]["rpl"]);
						$sb[$row]["public"] = "no";
					} else {
						rename("private/" . $sb[$row]["rpl"], $sb[$row]["rpl"]);
						$sb[$row]["public"] = "yes";
					}
					
					write2dini($tour["scoreboard"], $sb);
					break;
				case "d":
					PopLap($row, $tour["scoreboard"]);
					break;
				case "i":
					if (($sb[$row]["visible"] ?? "yes") == "no")
						$sb[$row]["visible"] = "yes";
					else
						$sb[$row]["visible"] = "no";
					
					write2dini($tour["scoreboard"], $sb);
					break;
			}
		}
	}
	
	if (($_GET["page"] ?? "") == "logout") {
		if (isset($_SESSION["user"])) unset($_SESSION["user"]);
		$isadmin = false;
	}
}


// ---------------------- LOAD STARTING FILES
$tour = readsimpleini("tour.cfg");
$tour["tour"] = $tour["tour"] ?? "Easy-Tournament";
$admin = readsimpleini("private/admin.cfg");
$admin["topimage"] = $admin["topimage"] ?? false;
$isadmin = (($_SESSION["user"]["name"] ?? "") == "admin");
?>

<html>
<head>
	<meta charset="UTF-8">
	<title><?=$tour["tour"]?></title>
	
	<link rel="stylesheet" type="text/css" href="easytour.css" />
	<?php
	if (file_exists("favicon.ico"))
		echo '<link rel="icon" href="favicon.ico">';
	?>
</head>
<body>
	<?php
	if (file_exists("topimage.png"))
		echo '<img src="topimage.png" id="topimage">';
	?>
	
	<h1 id="title-text"><?=$tour["tour"]?></h1>
	
	<?php
	// By default, admin password is "admin"
	if (($_SESSION["user"]["name"] ?? false) == false && !file_exists("private/admin.user")) {
		$adminuser["name"] = "admin";
		$adminuser["password"] = hash("snefru256", "admin");
		writesimpleini("private/admin.user", $adminuser);
	}
	
	reactions();
	menu();
	
	$page = $_GET["page"] ?? "home";
	switch ($page) {
		case "home":
			home_page();
			break;
		case "admin":
			admin_page();
			break;
		case "login":
			login_page();
			break;
		case "signup":
			signup_page();
			break;
		case "logout":
			echo "<p>Logged out</p>";
			break;
		case "profile":
			profile_page();
			break;
		case "rules":
			rules_page();
			break;
		default:
			echo "<p>Page not found</p>";
	}
	?>
</body>
</html>
